//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu>
// Creation Date: 23 January 1998
// Last Modified: Thu Feb 12 18:27:51 GMT-0800 1998
// Filename:      ...sig/examples/sigmidi/improv/synthImprov/sinus.cc
// Syntax:        C++; Synth Improv
//  
// Description: you press a key on the synthesizer keyboard, and a pitch 
//	pattern shaped like a sinewave is generated.  Attack velocity give 
//	amplitude of sine wave.  Alternatively, there is a synthesizer
//	keyboard mapped to the computer keyboard: the octave is controlled
//	by the numbers 0-9, and the chromatic pitches are on the bottom
//	two rows of the keyboard (e.g., 'z' is the note C).  '-' and '='
//	control the period size which controls how many notes to play
//	in one sinewave cycle.  '[' and ']' controls the tempo of the
//	sinewave notes in 100 millisecond increments.  If you play
//	a note loudly, then the sinewave pattern will be soft, and 
//	if you play a note softly, then the pattern will be loud.
//	The pattern is played as long as you are holding down a note.
//	If you are playing notes from the computer keyboard, then
//	their attack velocities are assigned random values from 1 to
//	127 since computer keyboards don't record attack velocity.
//

#include "synthImprov.h" 


/*----------------- beginning of improvization algorithms ---------------*/

#define TWOPI 6.283185307

int centernote = 0;          // played note which centers sinewave pattern
int amplitude = 0;           // greatest deviation amplitude of sine function
int sinenote = 0;            // current note of the sinewave
int sinevelocity = 0;        // current velocity of the sinewave
int period = 5;              // how many notes to play in the sinewave pattern
int count = 0;               // which note of the period we are on
int duration = 200;          // duration of sine notes
int octave = 4;              // octave range for computer keyboard notes
int keyboardnote = 0;        // computer keyboard note
int lastoff = 0;             // last note which was turned off
int nextnotetime = 0;        // time for the next note
MidiMessage noteMessage;     // for reading keyno and velocity (and time)



/*--------------------- maintenance algorithms --------------------------*/

void description(void) {
   cout << endl;
   printboxtop();
   psl(" Sinus -- by Craig Stuart Sapp <craig@ccrma.stanford.edu>, 24 Jan 1998");
   psl("");
   psl(" Description: press a key on the synthesizer keyboard, and a pitch");
   psl("     pattern shaped like a sinewave is generated.  Attack velocity gives");
   psl("     inverse amplitude of sine wave.  Computer keyboard keys are assigned");
   psl("     random attack velocities");
   printintermediateline();
   psl("  User commands:");
   psl("     \"-\" = decrease period length   \"=\" = increase period length ");
   psl("     \"[\" = decrease duration        \"]\" = increase duration");
   psl(" ");
   psl("  Computer synthesizer keyboard:");
   psl("     \"0\"-\"9\" = octave number of computer keyboard notes");
   psl("      Notes:           s   d      g    h   j   ");
   psl("                     z   x   c   v   b   n   m  ");
   printboxbottom();
   cout << endl;
} 



void initialization(void) { }


void finishup(void) { }


/*-------------------- main loop algorithms -----------------------------*/

void mainloopalgorithms(void) { 
   while (synth.getNoteCount() > 0) {
      count = 0;
      noteMessage = synth.extractNote();
      centernote = noteMessage.p1();
      if (noteMessage.p2() != 0)  sinevelocity = 127 - noteMessage.p2() + 20;
      else  sinevelocity = 0;
      if (sinevelocity > 127)  sinevelocity = 127;
      amplitude = midiscale(noteMessage.p2(), 0, 24);
      cout << "\r center = " << setw(4) << centernote
           << " amplitude = " << setw(4) << amplitude << "\t\t\t\t\t\t";
      cout.flush();
   }

   if (nextnotetime <= t_time) {
      nextnotetime = t_time + duration;
      if (lastoff != sinenote) {
         synth.play(0, sinenote, 0);       // turn off last note
         lastoff = sinenote;
      }
      sinenote = (int)(centernote + amplitude * 
         sin(TWOPI * count / period) + 0.5);
      if (sinevelocity != 0)  synth.play(0, sinenote, sinevelocity);
      count = (count + 1) % period;
   }
}
      


/*-------------------- triggered algorithms -----------------------------*/

void keyboardchar(int key) { 
   if (isdigit((char)key)) {
      octave = key - '0';   
      return;
   }
   switch (key) {
      case '-': 
         period--; 
         if (period < 1)  period = 1;
         cout << "\r period = " << period << "\t\t\t\t\t";
         cout.flush();
         break;
      case '=':
         period++; 
         if (period > 50)  period = 50;
         cout << "\r period = " << period << "\t\t\t\t\t";
         cout.flush();
         break;
      case '[':
         duration -= 25;
         if (duration < 25)  duration = 25;
         break;
      case ']':
         duration += 25;
         break;
      default:
         octavekeyboard(key, octave);
   }
}


/*------------------ end improvization algorithms -----------------------*/



