//
// Programmer:    Craig Stuart Sapp <craig@ccrma.stanford.edu> 
// Creation Date: 30 December 1997
// Last Modified: Mon Feb 16 00:23:11 GMT-0800 1998
// Filename:      ...sig/code/control/improv/batonImprov.h
// Syntax:        C++
//
// Description: The main function for radio baton improv programs.
//

#ifndef _BATONIMPROV_H_INCLUDED
#define _BATONIMPROV_H_INCLUDED


#include "improv.h"
#include <iostream.h>
#include <iomanip.h>
#include <fstream.h>
#include <stdlib.h>
#include <conio.h>          /* for kbhit() function */

#define REPORT_XYZ (777)

// function declarations
int  checkKeyboard(void);
int  chooseBatonInputDevice(void);
int  chooseBatonOutputDevice(void);
int  chooseSynthOutputDevice(void);
void finishup_automatic(void);
void displayBuffer(void);
void displayPositions(void);
void initialization_automatic(void);
void print_commands(void);
void print_aux_commands(void);
int  readmidiconfig(void);
void charsynth(int aKey);

// user defined functions
void description(void);
void keyboardchar(int key);
void initialization(void);
void mainloopalgorithms(void); 
void finishup(void);
void stick1trig(void);
void stick2trig(void);
void b14plustrig(void);
void b15plustrig(void);
void b14minusdowntrig(void);
void b14minusuptrig(void);
void b15minusdowntrig(void);
void b15minusuptrig(void);


////////////////////
// global variables

RadioDrum   baton;          // MIDI I/O for the radio drum
MidiOutput  synth;          // Midi Output for a synthesizer
Timer       mainTimer;      // Timer counting in milliseconds
long        t_time;         // global time (in milliseconds) which remains
                            // fixed for one pass through mainloopalgorithms()
char        recordFilename[1000] = {0};  // file to record Baton state



// Below are the global state variables for the baton which
// get updated as soon as a message command is received and
// before a complete set of trigger or position data is received.
//
// With these variables will not work if there are more than
// one Radio Drum.
//

// instantaneous position variables:
extern short pt1, pt2, pt3, pt4;             // current pot positions
extern short xt1, yt1, zt1, xt2, yt2, zt2;   // current stick positions

// trigger variables:
extern short xw1, yw1, xw2, yw2;             // whack positions of batons at trigger
extern short wh1, wh2;                       // whack strength of a trigger
extern short whack1, whack2;                 // booleans for triggers occuring
extern short whack1x, whack2x;               // booleans for triggers occuring
extern short whack1y, whack2y;               // booleans for triggers occuring
extern long  trigtime1, trigtime2;           // times that the sticks were triggered

// trigger plane variables:
extern short hit1, set1, hit2, set2;         // first four elements of calib array


///////////////////////////////////////////////////////////////////////////

int main(void) {
   int report = 0;       // boolean to display position information
   int report_type = 1;  // if 1 then reports xyz, else 0 reports buf
   int command;          // a keyboard single character command
   Timer displayTimer;
   displayTimer.setPeriod(200);  // timer when displaying buf data

   initialization_automatic();
   print_commands();   
   initialization();                     // user defined behavior

   while ((command = checkKeyboard()) != 'Q') {
      switch (command) {
         case 'B':                   // toggle display of buf
            if (report_type == REPORT_BUF) {
               report = !report;
            } else {
               report_type = REPORT_BUF;
               report = TRUE;
            }
            if (report) {
               displayTimer.reset();
            } else {
               cout << endl;
            }
            break;
         case 'P':                   // toggle display of positions
            if (report_type == REPORT_XYZ) {
               report = !report;
            } else {
               report_type = REPORT_XYZ;
               report = TRUE;
            }
            if (report) {
               displayTimer.reset();
            } else {
               cout << endl;
            }
            break;
         default: ;
            // nothing
      } // end switch (command)

      if (report && displayTimer.expired()) {
         displayTimer.reset();
         switch (report_type) {
            case REPORT_BUF:
               displayBuffer();
               break;
            case REPORT_XYZ:
               displayPositions();
               break;
         }
      }

      baton.processIncomingMessages();
      t_time = mainTimer.getTime(); 
      mainloopalgorithms();               // user defined behavior
   }

   finishup();                            // user defined behavior
   finishup_automatic();

   return 0;
}


///////////////////////////////////////////////////////////////////////////


//////////////////////////////
//
// checkKeyboard -- check to see if a key has been pressed on the keyboard.
//    If so, then interpret the keypress as a command.
//    Commands are listed in ASCII order.  Do not use the
//    characters: a-z, 0-9, or: ,./;'[]\=-`, control characters
//    or alt characters or the ESC key since these keys are
//    guarenteed to be available to the user for their own
//    use.
//

int checkKeyboard(void) {
   if (!kbhit()) return -1;

   char command;
   command = getch();

   switch (command) {
   // case ' ': break;
      case '!':                         // center stick 1
         baton.centerStick1();
         cout << "baton 2 centered at current x and y position." 
              << endl;
         break;
   // case '\"': break;
      case '#':                         // increase Z sensitivity
         baton.increaseZSensitivity();
         break;
      case '$':                         // decrease Z sensitivity
         baton.decreaseZSensitivity();
         break;
      case '%':                         // increase XY sensitivity
         baton.increaseXYSensitivity();
         break;
   // case '&': break;
   // case '(': break;
   // case ')': break;
   // case '*': break;
   // case '\'': break;
   // case '-': break;
   // case '.': break;
   // case '/': break;
   // case '0': break;
   // case '1': break;
   // case '2': break;
   // case '3': break;
   // case '3': break;
   // case '4': break;
   // case '5': break;
   // case '6': break;
   // case '7': break;
   // case '8': break;
   // case '9': break;
   // case ':': break;
   // case ';': break;
   // case '<': break;
   // case '=': break;
   // case '>': break;
      case '?':                         // print main command menu
         print_commands();
         break;
      case '@':                         // center stick 2
         baton.centerStick2();
         cout << "baton 2 centered at current x and y position." 
              << endl;
         break;
      case 'A':                         // baton alive test
         cout << "Radio Baton is ";
         cout.flush();
         if (baton.testBaton()) {
            cout << "alive." << endl;
         } else {
            cout << "dead." << endl;
         }
         break;
      case 'B':                         // toggle buf display 
         // This command is handled in the main() function
         break;
      case 'C':                         // send calibration data from file
         if (baton.readCalibrationFile("calib.def")) {
            cout << "Read calibration from file \".\\calib.def\"." 
                 << endl;
         } else if (baton.readCalibrationFile("..\\calib.def")) {
            cout << "Read calibration from file \"..\\calib.def\"." 
                 << endl;
         } else if (baton.readCalibrationFile("\\improv\\data\\calib.def")) {
            cout << "Read calibration from file \"\\improv\\data\\calib.def\"." 
                 << endl;
         } else if (baton.readCalibrationFile("c:\\improv\\data\\calib.def")) {
            cout << "Read calibration from file \"c:\\improv\\data\\calib.def\"." 
                 << endl;
         } else {
            cout << "Could not find calibration file for "
                 << "Radio Drum." << endl;
         }
         break;
      case 'D':                         // print program description
         // the description function is defined by
         // the user.  Hopefully they used it!
         description();
         break;
      case 'E':                         // set MIDI message error printing
         baton.toggleError();
         if (baton.getError()) {
            cout << "Radio Drum MIDI input error print is ON" << endl;
         } else {
            cout << "Radio Drum MIDI input error print is OFF" << endl;
         }
         break;
      case 'F':                      // set filename for baton state recording
         baton.pause();
         cout << "Type a filename for recording baton state variables: ";
         cin  >> recordFilename;
         baton.unpause();
         break;
   // case 'G': break; 
   // case 'H': break;
      case 'I':                         // set input port for Radio Drum
         cout << "radio drum MIDI input is currently set to device: " 
              << baton.getInputPort() << endl;
         baton.pause();
         baton.setInputPort(chooseBatonInputDevice());
         baton.openInput();
         cout << "radio drum input is now set to: "
              << baton.getInputPort() << endl;
         break;
   // case 'J': break;
      case 'K':                         // toggle record of baton state
         if (baton.recordingQ()) {
            cout << "Ending recording of baton data." << endl;
            baton.recordStateStop();
         } else {
            if (recordFilename[0] == '\0') {
               cout << "Start of baton data recording to file: " 
                    << "drum.rec" << endl;
               baton.recordStateStart("drum.rec");
            } else {
               cout << "Start of baton data recording to file: " 
                    << recordFilename << endl;
               baton.recordStateStart(recordFilename);
            }
         }
         break;
      case 'L':                         // set stick levels
         baton.setStickLevels(); 
         cout << "Trigger levels of batons set to current positions." 
              << endl;
         break;
      case 'M':                         // display more commands
         // this command displays less important commands
         print_aux_commands();
         break;
      case 'N':                         // set MIDI out device for synth
         cout << "synthesizer MIDI output is currently set to device: "
              << synth.getPort() << endl;
         synth.setPort(chooseSynthOutputDevice());
         synth.open();
         cout << "synthesizer MIDI output is now set to: "
              << synth.getPort() << endl;
         break;
      case 'O':                         // set MIDI out device for radio drum
         cout << "radio drum MIDI output is currently set to device: "
              << baton.getOutputPort() << endl;
         baton.setOutputPort(chooseBatonOutputDevice());
         baton.openOutput();
         cout << "radio drum MIDI output is now set to: "
              << baton.getOutputPort() << endl;
         break;
      case 'P':                         // toggle stick and pot reporting
         // this command is handled in the main() function.
         break;
      case 'Q':                         // quit the program
         // don't do anything here, the calling function (main)
         // will handle this command.
         break;
      case 'R':                         // toggle position reporting
         baton.togglePositionReporting();
         if (baton.getPositionReporting()) {
            cout << "Position Reporting turned ON." << endl;
         } else {
            cout << "Position Reporting turned OFF." << endl;
         }
         break;
      case 'S':                         // send note-offs to the synth
         cout << "Silencing synthesizer ...";
         synth.silence();
         cout << " done." << endl;
         break;
   // case 'T': break;
   // case 'U': break;
   // case 'V': break;
      case 'W':                         // get calibration data and save to file
         baton.writeCalibrationFile("calib.def");
         break;
      case 'X':                         // toggle MIDI output trace
         baton.toggleOutputTrace();
         if (baton.getOutputTrace()) {
            cout << "MIDI output trace is ON." << endl;
         } else {
            cout << "MIDI output trace is OFF." << endl;
         }
         synth.setTrace(baton.getOutputTrace());
         break;
      case 'Y':                         // toggle MIDI input trace
         baton.toggleInputTrace();
         if (baton.getInputTrace()) {
            cout << "MIDI input trace is ON." << endl;
         } else {
            cout << "MIDI input trace is OFF." << endl;
         }
         break;
   // case 'Z': break;
   // case '[': break;
   // case '\': break;
   // case ']': break;
      case '^':                         // decrease XY sensitivity
         baton.decreaseXYSensitivity();
         break;
   // case '_': break;
   // case '`': break;
   // case 'a': break;
   // case 'b': break;
   // case 'c': break;
   // case 'd': break;
   // case 'e': break;
   // case 'f': break;
   // case 'g': break;
   // case 'h': break;
   // case 'i': break;
   // case 'j': break;
   // case 'k': break;
   // case 'l': break;
   // case 'm': break;
   // case 'n': break;
   // case 'o': break;
   // case 'p': break;
   // case 'q': break;
   // case 's': break;
   // case 't': break;
   // case 'u': break;
   // case 'v': break;
   // case 'w': break;
   // case 'x': break;
   // case 'y': break;
   // case 'z': break;
   // case '{': break;
   // case '|': break;
   // case '}': break;
   // case '~': break;
      default:                          // unknown command
         // can't identify command, so it must be a user command.
         // give the character to the user command:
         keyboardchar(command);
   } // end of switch (command)

   return command;
}
   


//////////////////////////////
//
// chooseBatonInputDevice -- have the user select the 
//    MIDI input device to communicate with the radio drum.
//

int chooseBatonInputDevice(void) {
   baton.pause();
   int deviceCount = baton.getNumInputPorts();

   if (deviceCount <= 1) {
      baton.unpause();
      return 0;
   }

   int choice = -100;

   cout << "Choose one of the following MIDI input ports "
        << "for Baton communication:" << endl;
   displayInputDevices();

   while (choice < 0 || choice >= deviceCount) {
      cout << "Type port number choice here: ";
      cin  >> choice;
   }

   baton.unpause();
   return choice;
}



//////////////////////////////
//
// chooseBatonOutputDevice -- have the user select the 
//    MIDI output device to communicate with the radio drum.
//

int chooseBatonOutputDevice(void) {
   baton.pause();
   int deviceCount = baton.getNumOutputPorts();

   if (deviceCount <= 1) {
      baton.unpause();
      return 0;
   }

   int choice = -100;

   cout << "Choose one of the following MIDI output ports "
        << "for Baton communication:" << endl;
   displayOutputDevices();

   while (choice < -1 || choice >= deviceCount) {
      cout << "Type port number choice here: ";
      cin  >> choice;
   }

   baton.unpause();
   return choice;
}



//////////////////////////////
//
// chooseSynthOutputDevice -- have the user select the 
//    MIDI output device to communicate with the synthesizer.
//

int chooseSynthOutputDevice(void) {
   baton.pause();
   int deviceCount = baton.getNumOutputPorts();

   if (deviceCount <= 1) {
      baton.unpause();
      return 0;
   }

   int choice = -100;

   cout << "Choose one of the following MIDI output ports "
        << "for synthesizer communication:" << endl;
   displayOutputDevices();

   while (choice < -1 || choice >= deviceCount) {
      cout << "Type port number choice here: ";
      cin  >> choice;
   }

   baton.unpause();
   return choice;
}



//////////////////////////////
//
// displayBuffer -- display the buf data from the baton
//
void displayvalue(int buf, const char* spacer) {
   if (buf == -1){
      cout << "XXXX";
   } else {
      cout << setw(4) << buf;
   }
   cout << spacer;
}

void displayBuffer(void) {
   cout << "\r";
   displayvalue(baton.getBuf(0), " ");
   displayvalue(baton.getBuf(1), " ");
   displayvalue(baton.getBuf(2), " ");
   displayvalue(baton.getBuf(3), " ");
   displayvalue(baton.getBuf(4), ":");
   displayvalue(baton.getBuf(5), ":");
   displayvalue(baton.getBuf(6), " ");
   displayvalue(baton.getBuf(7), " ");
   displayvalue(baton.getBuf(8), " ");
   displayvalue(baton.getBuf(9), " ");
   displayvalue(baton.getBuf(10), ":");
   displayvalue(baton.getBuf(11), " ");
   displayvalue(baton.getBuf(12), " ");
   displayvalue(baton.getBuf(13), ":");
   displayvalue(baton.getBuf(14), " ");
   displayvalue(baton.getBuf(15), "");

   cout.flush();
   baton.requestBufData();
}



//////////////////////////////
//
// displayPositions -- display the current positions of the
//   batons, dials, etc.
//
 
void displayPositions(void) {
   cout << "\r";
   cout << "Stick1: " 
        << setw(4) << (int)baton.x1p[0] << ","
        << setw(4) << (int)baton.y1p[0] << ","
        << setw(4) << (int)baton.z1p[0] << ".   "
        << "Stick2: "
        << setw(4) << (int)baton.x2p[0] << ","
        << setw(4) << (int)baton.y2p[0] << ","
        << setw(4) << (int)baton.z2p[0] << ".   "
        << "Dials: "
        << setw(4) << (int)baton.d1p[0] << ","
        << setw(4) << (int)baton.d2p[0] << ","
        << setw(4) << (int)baton.d3p[0] << ","
        << setw(4) << (int)baton.d4p[0]
       ;
   cout << " ";
   cout.flush();
}



//////////////////////////////
//
// finishup_automatic -- place automatic configuration 
//    commands here.
//

void finishup_automatic(void) {
   cout << endl;
}

     

//////////////////////////////
//
// initialization_automatic -- place automatic initialization
//    commands here.
//

void initialization_automatic(void) {

   // choose the MIDI in and out ports
   if (readmidiconfig() == 0) {   
      baton.setInputPort(chooseBatonInputDevice());
      baton.openInput();
      baton.setOutputPort(chooseBatonOutputDevice());
      baton.openOutput();
      synth.setPort(chooseSynthOutputDevice());
      synth.open();
   }

   // try to calibrate the radio drum automatically
   if (baton.readCalibrationFile(".\\calib.def")) {
      cout << "Reading calibration from \".\\calib.def\"." << endl;
   } else if (baton.readCalibrationFile("..\\calib.def")) {
      cout << "Reading calibration from \"..\\calib.def\"." << endl;
   } else if (baton.readCalibrationFile("c:\\improv\\data\\calib.def")) {
      cout << "Reading calibration from \"c:\\improv\\data\\calib.def\"." 
           << endl;
   } else {
      cout << "Could not find calibration file.  "
           << "Radio Drum not calibrated." << endl;
      baton.requestCalibration();
   }

   baton.positionReportingOn();     // have the baton send position data
}



//////////////////////////////
//
// print_commands -- print keyboard character commands
//

void print_commands(void) {
   cout << endl;
   printboxtop();
   pcl(
"UPPERCASE COMMANDS :: Improv Interface :: Music 120 :: Winter 1998");
   printintermediateline();
   psl("   D = print a description of this program given by the programmer");
   psl("   ? = print this command list         M = print more commands");
   psl("   Q = quit the program                A = baton alive test");
   psl("   L = set stick levels                S = silence synthesizer");
   psl("   shift-1 = center stick1             shift-2 = center stick2");
   psl("   shift-3 = increase Z sensitivity    shift-4 = decrease Z sensitivity");
   psl("   shift-5 = increase XY sensitivity   shift-6 = decrease XY sensitivity");
   psl("   P = position display toggle         B = buf display toggle");
   psl("   W = write calibration of batons     C = calibrate batons");
   printboxbottom();
}



void print_aux_commands(void) {
   cout << endl;
   printboxtop();
   pcl("Radio Drum Improv Interface version 12/30/97 by");
   pcl("Craig Stuart Sapp  <craig@ccrma.stanford.edu>");
   printintermediateline();
   psl("   I = set MIDI in port for batons     E = radio drum MIDI error toggle");
   psl("   O = set MIDI out port for batons    N = set MIDI out port for synth");
   psl("   R = toggle position reporting");
   psl("   X = toggle MIDI out trace           Y = toggle MIDI in trace");
   psl("   K = toggle drum state recording     F = set drum state recording filename");
   printboxbottom();
}



//////////////////////////////
//
// readmidiconfig -- returns true if it found the midiconfig file
//    which stores the input and output device numbers of the
//    computer for the programs.  first number in file
//    is the baton input device, second number in the file is
//    the baton output device, and the third number in the file
//    is the synth output device. 
//

int readmidiconfig(void) {
   int batonin, batonout, synthout;
   ifstream infile1, infile2, infile;

   infile1.open("c:\\improv\\data\\midi.def", ios::nocreate);
   infile2.open(".\\midi.def", ios::nocreate);

   // had to do it this for some buggy reason:
   if (infile1.is_open()) {
      cout << "Midi configuration read from \"c:\\improv\\data\\midi.def\"" 
           << endl;
      infile1.close();
      infile.open("c:\\improv\\data\\midi.def", ios::nocreate);
   } else if (infile2.is_open()) {
      cout << "Midi configuration read from \".\\midi.def\"" << endl;
      infile2.close();
      infile.open(".\\midi.def", ios::nocreate);
   } else {
      return 0;
   }

   if (!infile.is_open()) {
      return 0;
   } else {
      infile >> batonin;
      infile >> batonout;
      infile >> synthout;

      baton.setInputPort(batonin);
      baton.openInput();
      baton.setOutputPort(batonout);
      baton.openOutput();
      synth.setPort(synthout);
      synth.open();

      cout << "Baton MIDI input set to device " << batonin << " = "
         << baton.MidiInput::getName() << endl;
      cout << "Baton MIDI output set to device " << batonout << " = "
         << baton.MidiOutput::getName() << endl;
      cout << "Synthesizer output set to device " << synthout << " = "
         << synth.getName() << endl;

      return 1;
   }
}




//////////////////////////////
//
// charsynth -- plays the synthesizer with lowercase characters
//    on the computer's keyboard (bottom two rows).
// z=C4, x=D4, c=E4, v=F4, b=G4, n=A4, m=B4, ","=C5, etc.
//

void charsynth(int aKey) {
   static MidiMessage noteMessage;
   static int attack = 0;
   static int note = 0;

   synth.play(CH_1, note, 0);

//   noteMessage.time = mainTimer.getTime();
//   noteMessage.command() = 0x90;
//   noteMessage.p1() = note;
//   noteMessage.p2() = 0;
//   synth.insert(noteMessage);

   switch (aKey) {
      case 'z':  note=C4;    break;
      case 'x':  note=D4;    break;
      case 'c':  note=E4;    break;
      case 'v':  note=F4;    break;
      case 'b':  note=G4;    break;
      case 'n':  note=A4;    break;
      case 'm':  note=B4;    break;
      case ',':  note=C5;    break;
      case '.':  note=D5;    break;
      case '/':  note=E5;    break;
      case 'a':  note=B3;    break;
      case 's':  note=Cs4;   break;
      case 'd':  note=Eb4;   break;
      case 'f':  note=0;     break;
      case 'g':  note=Fs4;   break;
      case 'h':  note=Gs4;   break;
      case 'j':  note=Bb4;   break;
      case 'k':  note=0;     break;
      case 'l':  note=Cs5;   break;
      case ';':  note=Eb5;   break;
      case '\'': note=Fs5;   break;
      default: return;
   }

   if (note < 0)  note = 0;
   else if (note > 127)  note = 127;

   attack = rand()%47 + 81;           // random int from 1 to 127
   synth.play(0,note,attack); 

//   noteMessage.time = mainTimer.getTime();
//   noteMessage.command() = 0x90;
//   noteMessage.p1() = note;
//   noteMessage.p2() = rand()%47 + 81;      // random int from 1 to 127
//   synth.insert(noteMessage);

}


#endif  // _BATONIMPROV_H_INCLUDED

